* {
  background: transparent;
  border: 0;
  font-size: 100%;
  margin: 0;
  padding: 0;
  vertical-align: baseline;
}

html {
  background-color: #fff;
  background-image: url(../media/bg-header.png);
  background-position: left top;
  background-repeat: repeat-x;
}

html, body {
  height: 100%;
}

body {
  color: #444;
  font-size: 13px;
  line-height: 1.538em;
  margin: 0 auto;
  min-width: 780px;
  position: relative;
  font-family: "helvetica neue", arial, helvetica, sans-serif;
  font-style: normal;
}

table {
  width: 100%;
  border-collapse: collapse;
}

table, td, tr {
  border: 1px black solid;
}

td {
  padding: 0.5em;
}

/* Rule to fix quirks-mode inheritance behavior */

table, caption {
  font-size: inherit;
  font-weight: inherit;
  font-style: inherit;
  font-variant: inherit;
}

table.meta-table, .meta-table td, .meta-table tr {
  vertical-align: top;
  border: none;
  padding: 0;
  padding-top: 0.25em;
  padding-bottom: 0.25em;
  padding-right: 5em;
}

table.meta-table {
  margin-top: 0.5em;
  border-top: 1px dotted gray;
  border-bottom: 1px dotted gray;
}

a:link {
  color: #003595;
  text-decoration: none;
}

a:visited {
  color: #003595;
  text-decoration: none;
}

a:hover, a:focus, a:active {
  color: #003595;
  text-decoration: underline;
}

pre, code, kbd, tt, samp, tt {
  font-family: "andale mono", monospace;
}

h1 {
  font-size: 2.571em;
  line-height: 1em;
  margin-bottom: .5833em;
  margin-top: .1667em;
  font-family: Georgia, serif;
  font-weight: normal;
}

h2 {
  color: #011234;
  font-size: 2.154em;
  font-weight: normal;
  line-height: 1em;
  margin-bottom: .5em;
  margin-top: .5em;
  font-family: Georgia, serif;
  font-style: italic;
}

h3 {
  color: #011234;
  font-size: 1.385em;
  font-weight: normal;
  line-height: 1.111em;
  margin-bottom: .778em;
  margin-top: 1.556em;
  font-family: Georgia, serif;
  font-style: italic;
}

h4 {
  color: #235;
  font-size: 1em;
}

h5, h6 {
  font-size: 1em;
}

li {
  margin-left: 2em;
}

ol {
  list-style-type: decimal;
  margin-left: 2em;
}

dt {
  font-weight: bold;
  margin-top: 1.5em;
}

ul ul, ol ol, ol ul, ul ol {
  margin-bottom: 0;
}

p, ul, ol, dl, address, table, pre, form, fieldset {
  font-size: 1em;
  margin-bottom: 1.5em;
}

th, td {
  text-align: left;
}

@font-face {
  font-family: 'MetaBlack';
  font-style: normal;
  font-weight: bold;
  src: url('https://www.mozilla.com/img/fonts/MetaWebPro-Black.woff') format('woff');
}
