html, body {
  min-width: 570px;         /* 2 x sidebar fullwidth + main-content padding */
  height: 100%;
}

#columns {
  padding-left: 250px;      /* sidebar width + sidebar padding */
  padding-right: 65px;     /* main-content padding*/
  padding-top: 40px;
  min-height: 100%;
  margin-bottom: 170px;
  position: relative;
}

.funnel .menu {
  padding-right: 50px;     /* main-content padding*/
}

#mozilla-tab {
  padding-right: 50px;     /* main-content padding*/
}

#site-header h1 a span {
  top: 25px;
  left: 45px;
}

#site-header h1 a {
  padding-left: 45px;
}

#columns .column {
  position: relative;
  float: left;
}

#main-content-column {
  width: 100%;
  padding-right: 50px;
}

.check, .cross {
 text-align: center;
}

#sidebar {
  width: 200px;        /* sidebar usable width (fullwidth - padding) */
  right: 300px;        /* sidebar fullwidth + main-content padding */
  padding-left: 30px;
  padding-right: 20px;
  margin-left: -100%;
}

.api_reference h3, .api_reference  h4,
.api_reference  h5, .api_reference  h6 {
  overflow: auto;
 }

#toc {
  width: 180px;
  float: right;
  font-size: 0.9em;
  overflow: hidden;
  margin-top: 0;
  margin-left: 20px;
  margin-bottom: 1em;
}

#toc a {
  font-family: Georgia, serif;
  font-weight: normal;
  font-style: italic;
}

.title {
  height: 30px;
  margin-bottom: 10px;
}

.sidebar-section, #toc {
  border-style: solid;
  border-color: #a0d0fb;
  border-width: 1px;
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  -khtml-border-radius: 5px;
  border-radius: 5px;
  background: #e0effd;
  margin-bottom: 1.0em;
}

.aside {
  color: gray;
  float: right;
  padding-left: 1em;
  margin-left: 1em;
  margin-right: 0;
  width: 25%;
  border-left: 1px solid gray;
}

#toc a {
  display: block;
  }

#toc a[class = H2] {
  font-size: 1.1em;
  font-weight: bold;
  margin-left: 0.5em;
}

#toc a[class = H3] {
  margin-left: 0.5em;
  font-weight: bold;
}

#toc a[class = H4] {
  margin-left: 1.5em;
  font-family: "andale mono", monospace;
  font-style: normal;
}

#toc a[class = H5] {
  margin-left: 2.5em;
  font-weight: bold;
}

#toc a[class = H6] {
  margin-left: 3em;
  font-family: "andale mono", monospace;
  font-style: normal;
}

.warning {
  margin-left: auto;
  margin-right: auto;
  margin-top: 2em;
  margin-bottom: 2em;
  display: block;
  padding: 1em;
  border: solid 2px black;
  width: 75%;
  background-color: #c63717;
  color: white;
}

.logo {
  margin-left: 5px;
  float: left;
}

.meta-header {
  font-weight: bold;
}

.sidebar-section .modules {
  padding-left: 25px;
}

.package-entry .modules
.package-detail .modules {
  margin: 0em;
  padding-left: 0;
}

.module {
  margin-top: 0.2em;
  margin-bottom: 0.2em;
  padding-left: 0em;
  padding-right: 0em;
  margin-right: 1.0em;
}

.package-summary .module {
  padding-left: 0em;
}

.package-summary .module,
.package-entry .module,
.package-detail .module {
  display: inline;
  white-space: nowrap;
  margin-left: 0em;
}

.package-detail .authors, .package-detail .dependencies,
.package-detail .version, .package-detail .license {
  font-family: Monaco, Lucida Console, monospace;
  font-size: 0.8em;
  white-space: pre;
}

#main-content h1 {
  font-size: 2.2em;
}

#main-content h2 {
  font-size: 1.7em;
}

.sidebar-subsection-contents {
  display: none;
}

.sidebar-subsection {
  margin: 1px;
}

.always-show {
  display: normal;
}

.sidebar-section ul {
  margin-bottom: 0.2em;
  list-style: none;
}

.sidebar-section li li {
  padding-left: 30px;
  margin-left: 0;
}

.sidebar-section .modules {
  padding-left: 25px;
}

.sidebar-section h2, .sidebar-section h4 {
  padding-left: 10px;
}

.sidebar-section h3 {
  padding-left: 5px;
}

.sidebar-section h2, .sidebar-section h3 {
  padding-top: 5px;
  padding-bottom: 5px;
  margin: 0px;
  font-style: normal;
  font-family: arial, helvetica, sans-serif;
}

.sidebar-section h2 {
  background-color: #3b69b1;
  font-family: Georgia, serif;
  font-style: italic;
  font-size: 1.4em;
  -moz-border-radius-topleft: 5px;
  -moz-border-radius-topright: 5px;
  -webkit-border-radius-topleft: 5px;
  -webkit-border-radius-topright: 5px;
  -khtml-border-radius-topleft: 5px;
  -khtml-border-radius-topright: 5px;
  border-radius-topleft: 5px;
  border-radius-topright: 5px;
}

.sidebar-section h2 a {
  color: white;
}

.sidebar-section h2.current-page a {
  color: #3b69b1;
}

.sidebar-section h3 {
  font-family: Georgia, serif;
  font-style: italic;
  font-size: 1.2em;
}

.divider {
  border-style: solid;
  border-top-width: 1px;
  border-color: #a0d0fb;
  margin-left: 5px;
  margin-right: 5px;
}

.sidebar-section h4 {
  font-weight: normal;
  font-family: Georgia, serif;

}

.sidebar-section li {
  font-family: Georgia, serif;
  margin-left: 0;
}

.sidebar-section .current-page {
  background-color: white;
}

.image-center {
  margin-left: auto;
  margin-right: auto;
  display: block;
}
